<?php

namespace Iwaydev\LaravelLogger;

use Illuminate\Support\ServiceProvider;

/**
 * Провайдер сервиса переводов
 *
 * @package Iwaydev\Localization,
 */
class LogServiceProvider extends ServiceProvider
{
    /**
     * Register services.
     *
     * @return void
     */
    public function register(): void
    {
        parent::register();
        $this->loadHelpers();
    }

    /**
     * Подгружаем хэлперы
     *
     * @return void
     */
    protected function loadHelpers(): void
    {
        foreach (glob(__DIR__ . '/Helpers/*.php') as $filename) {
            require_once $filename;
        }
    }

    /**
     * Добавляет в конфиг канал для логирования, если отсутствует
     */
    protected function configMerge(): void
    {
        if (!config()->has('logging.request_response')) {
            config()->set('logging.request_response', require __DIR__ . '/../config/request_response.php');
        }

        if (!config()->has('logging.stream')) {
            config()->set('logging.stream', require __DIR__ . '/../config/stream.php');
        }

        if (config('logging.stream.enable')) {
            foreach (config()->get('logging.channels') as $channelName => $channelConfig) {
                if (in_array($channelName, config('logging.stream.exclude_channels'))) {
                    continue;
                }
                config()->set(
                    'logging.channels.' . $channelName,
                    array_merge(
                        $channelConfig,
                        [
                            'driver' => 'monolog',
                            'handler' => config('logging.stream.handler'),
                            'with' => ['stream' => 'php://stderr'],
                            'formatter' => config('logging.stream.formatter'),
                            'formatter_with' => [
                                'serverName' => config('logging.stream.server_name'),
                                'projectName' => config('app.name'),
                                'logCategory' => $channelName,
                                'tracking' => config('logging.request_response.tracking'),
                                'paramsMasking' => config('logging.request_response.params_masking'),
                                'entryPoint' => $this->app->runningInConsole() ? 'console' : ''
                            ],
                        ]
                    )
                );
            }
        }
    }

    /**
     * Bootstrap services.
     *
     * @return void
     */
    public function boot(): void
    {
        $this->configMerge();
        $sourceRequestResponseConfig = realpath(
            $rawConfig = __DIR__ . '/../config/request_response.php'
        ) ?: $rawConfig;
        $sourceStreamConfig = realpath($rawConfig = __DIR__ . '/../config/stream.php') ?: $rawConfig;
        if ($this->app->runningInConsole()) {
            $this->publishes(
                [
                    $sourceRequestResponseConfig => $this->app->configPath('logging/request_response.php'),
                    $sourceStreamConfig => $this->app->configPath('logging/stream.php'),
                ],
                'config'
            );
        }
    }
}
