<?php

namespace Iwaydev\LaravelLogger\Traits;

use Psr\Log\LoggerInterface;
use Stringable;
use Throwable;

trait LoggerTrait
{
    private ?LoggerInterface $logger = null;
    private string $logMessagePrefix = '';

    /**
     * @param LoggerInterface|null $logger
     * @param string $messagePrefix
     *
     * @return void
     */
    public function setLogger(?LoggerInterface $logger, string $messagePrefix = ''): void
    {
        $this->logger = $logger;
        $this->logMessagePrefix = $messagePrefix;
    }

    /**
     * @param string|Stringable $message
     *
     * @return string
     */
    private function prepareMessage(string|Stringable $message): string
    {
        $prefix = config('logging.stream.enable') ? '' : $this->logMessagePrefix;

        return trim($prefix . ' ' . $message);
    }

    /**
     * @param string|Stringable $message
     * @param mixed[] $context
     *
     * @return void
     */
    private function logInfo(string|Stringable $message, array $context = []): void
    {
        $this->logger?->info($this->prepareMessage($message), $context);
    }

    /**
     * @param string|Stringable $message
     * @param mixed[] $context
     *
     * @return void
     */
    private function logWarning(string|Stringable $message, array $context = []): void
    {
        $this->logger?->warning($this->prepareMessage($message), $context);
    }

    /**
     * @param string|Stringable $message
     * @param mixed[] $context
     *
     * @return void
     */
    private function logError(string|Stringable $message, array $context = []): void
    {
        $this->logger?->error($this->prepareMessage($message), $context);
    }

    /**
     * @param Throwable $t
     * @param array $context
     *
     * @return void
     */
    public function logException(Throwable $t, array $context = []): void
    {
        if (config('logging.stream.enable')) {
            $this->logError(
                'Исключение! ' . $t->getMessage(),
                array_merge(
                    $context,
                    [
                        'exception' => [
                            'error_code' => $t->getCode(),
                            'file' => $t->getFile() . '(' . $t->getLine() . ')',
                            'trace' => collect($t->getTrace())->take(10)->toJson()
                        ]
                    ]
                )
            );
        } else {
            $this->logError('');
            $this->logError('Исключение!');
            $this->logError('Сообщение: ' . $t->getMessage(), $context);
            $this->logError('Код: ' . $t->getCode());
            $this->logError('Файл: ' . $t->getFile() . '(' . $t->getLine() . ')');
            $this->logError('Трейс: ' . PHP_EOL . $t->getTraceAsString());
            $this->logError('');
        }
    }
}
